<?php

/**
 * @file
 *  The UI page for the master module and all the dependent submit functions.
 */


/**
 * Form callback for master control.
 *
 * This provides the form to enabled, disable and revert features and modules.
 *
 * @param array $form_state
 *   the form_state array known from drupal_get_form() ...
 * @return array
 *   a Form API array
 */
function master_modules_overview($form_state = array()) {
  $form = array();

  $modules = module_rebuild_cache();
  $masters = master_get_modules();
  $all_dependencies = array();
  $non_dependent = '';

  // Provide fieldset for each master module first.
  $form['masters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Master modules (@count)', array('@count' => count($masters))),
    '#description' => t('These are the defined master modules.'),
    '#weight' => 0,
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $master_modules = array_intersect_key($modules, array_flip($masters));
  $form['masters']['modules'] = _master_modules_overview_masters_tableselect_form($master_modules, array('masters'));

  // The basic dependent fieldset
  $form['dependent'] = array(
    '#type' => 'fieldset',
    '#title' => t('Dependencies'),
    '#collapsible' => TRUE,
    '#weight' => 1,
    '#descriptions' => t('Enable disabled but dependent modules.'),
  );
  $form['dependent']['enable'] = array(
    '#type' => 'submit',
    '#value' => t('Enable dependent modules'),
    '#submit' => array('master_modules_enable_submit'),
    '#weight' => 99,
  );


  // Build basic Features fieldset, if features is enabled.
  if (module_exists('features')) {
    $features = features_get_features();
    module_load_include('inc', 'features', 'features.export');
    features_include();

    // The basic dependent fieldset
    $form['features'] = array(
      '#type' => 'fieldset',
      '#title' => t('Features'),
      '#collapsible' => TRUE,
      '#weight' => 3,
      '#descriptions' => t('Use the checkboxes to revert selected features of the given masters.'),
    );
    $form['features']['features_revert'] = array(
      '#type' => 'submit',
      '#value' => t('Revert selected features'),
      '#submit' => array('master_modules_features_revert_submit'),
      '#weight' => 99,
    );
  }

  // Run through each master and fill the 'masters', 'dependent' and 'features'
  // fieldsets.
  $any_master_disabled = FALSE;
  $all_inactive_dependencies_count = 0;
  $all_features_count = 0;
  foreach ($master_modules as $master) {
    // Overall dependencies of the given master.
    $dependencies = $master->info['dependencies'];
    $dependency_count = count($dependencies);

    // Provide the dependent selection.
    $form['dependent'][$master->name] = array (
      '#type' => 'fieldset',
      '#title' => check_plain($master->info['name']) .' ('. format_plural($dependency_count, '1 dependency', '@count dependencies', array('@count' => $dependency_count)) .')',
      '#weight' => 1,
      '#collapsible' => TRUE,
    );
    // We only need to show further details, if there are dependencies.
    if (!$dependency_count) {
      $form['dependent'][$master->name]['#description'] = t('No dependencies for %master.', array('%master' => $master->info['name']));
    }
    // There are dependencies, so we check each and provide checkboxes.
    else {
      $dependent_modules = array_intersect_key($modules, array_flip($dependencies));
      $form['dependent'][$master->name]['#description'] = t('These modules are dependencies of the master module %master.', array('%master' => $master->name));
      $form['dependent'][$master->name]['modules'] = _master_modules_overview_modules_tableselect_form($dependent_modules, FALSE, array('dependent', 'modules'));
      $form['dependent'][$master->name]['modules']['#empty'] = t('No available disabled dependent modules detected.');

      // List non available modules.
      $non_available_dependencies = array_diff($dependencies, array_keys($dependent_modules));
      if (count($non_available_dependencies)) {
        $list = implode(', ', $non_available_dependencies);
        $form['dependent'][$master->name]['non_available'] = array(
          '#prefix' => '<span class="warning">',
          '#value' => t('Some modules are not available, please download them and check again: %list', array('%list' => $list)),
          '#suffix' => '</span>',
        );
      }

      // Collapse if there are no modules.
      $count = count(element_children($form['dependent'][$master->name]['modules']));
      if (!$count) {
        $form['dependent'][$master->name]['#collapsed'] = TRUE;
      }
      $all_inactive_dependencies_count += $count;
    }

    // Provide the features part, if Features is enabled.
    if (module_exists('features')) {
      // Provide the dependent selection.
      $form['features'][$master->name] = array (
        '#type' => 'fieldset',
        '#title' => check_plain($master->info['name']),
        '#collapsible' => TRUE,
      );
      $dependent_features = array_intersect_key($features, array_flip($dependencies));
      $form['features'][$master->name]['modules'] = _master_modules_overview_features_tableselect_form($dependent_features, array('features', 'modules'));
      $count = count(element_children($form['features'][$master->name]['modules']));
      $form['features'][$master->name]['#title'] .= " ($count)";
      // Collapse if there are no modules.
      if (!$count) {
        $form['features'][$master->name]['#collapsed'];
      }
      $all_features_count += $count;
    }

    // Collect all dependencies for independent modules below.
    $all_dependencies += $dependencies;
  }
  $all_dependencies = array_unique($all_dependencies);

  // Do not collapse the master fieldset, if a master module has to be activated.
  if ($any_master_disabled) {
    $form['masters']['#collapsed'] = FALSE;

    // And add the button for enabling.
    $form['masters']['enable'] = array(
      '#type' => 'submit',
      '#value' => t('Enable masters'),
      '#submit' => array('master_modules_enable_masters_submit'),
      '#weight' => 99,
    );
  }
  // Collapse if there are no inactive dependent modules at all.
  $all_dependencies_count = count($all_dependencies);
  $form['dependent']['#title'] .= " ($all_inactive_dependencies_count / $all_dependencies_count)";
  if (!$all_inactive_dependencies_count) {
    $form['dependent']['#collapsed'] = TRUE;
  }
  // Else add the submit button.
  else {

  }
  // Collapse if there are no dependent features at all.
  $form['features']['#title'] .= " ($all_features_count)";
  if (!$all_features_count) {
    $form['features']['#collapsed'] = TRUE;
  }

  // Run through not dependent but enabled modules, to provide opportunity to
  // disable them directly.
  $independent_modules = array_diff_key($modules, array_flip($all_dependencies), array_flip($masters), array('master' => 1));
  $independent_modules = array_filter($independent_modules, '_master_array_filter_module_enabled');
  $independent_modules_count = count($independent_modules);
  $form['independent'] = array(
    '#type' => 'fieldset',
    '#title' => t('Independent modules (@count)', array('@count' => $independent_modules_count)),
    '#collapsible' => TRUE,
    '#collapsed' => !$independent_modules_count,
    '#weight' => 2,
  );
  $form['independent']['modules'] = _master_modules_overview_modules_tableselect_form($independent_modules, TRUE, array('independent', 'modules'));
  $form['independent']['modules']['#weight'] = 1;
  $form['independent']['modules']['#empty'] = t('No enabled independent modules detected.');
  // Add button, if there are modules to disable.
  if ($independent_modules_count) {
    $form['independent']['disable'] = array(
      '#type' => 'submit',
      '#value' => t('Disable independent modules'),
      '#submit' => array('master_modules_disable_submit'),
      '#weight' => 99,
    );
  }
  else {

  }

  return $form;
}

/**
 * Provide a checkable tabelselect list of master modules.
 *
 * @param array $modules
 *   array of (master)module objects
 * @param array $parents
 *   array of additional parents to set to the form elements.
 * @return array
 *   form array with checkboxes
 */
function _master_modules_overview_masters_tableselect_form($modules, $parents = array()) {

  // Provide the master list als tableselect.
  $form = array(
    '#theme' => 'master_tableselect',
    '#header' => array('', t('Module'), t('Status'), t('Description')),
    '#columns' => array('checkbox', 'name', 'status', 'description'),
    '#empty' => t('No masters'),
  );

  foreach ($modules as $module) {
    // Provide checkboxes and description for inactive modules.
    if (!$module->status) {
      $form[$module->name]['checkbox'] = array(
        '#type' => 'checkbox',
        '#parents' => array_merge($parents , array($module->name)),
      );
      $any_master_disabled = TRUE;
      $form['masters']['#collapsed'] = FALSE;
    }
    $form[$module->name]['name'] = array(
      '#value' => '<strong>'. check_plain($module->info['name']) .'</strong>',
    );
    $form[$module->name]['status'] = array(
      '#value' => ($module->status) ? t('Enabled') : t('Disabled'),
    );
    $form[$module->name]['description'] = array(
      '#value' => filter_xss_admin($module->info['description']),
    );
  }
  return $form;
}

/**
 * Provide a checkable tabelselect list of modules.
 *
 * @param array $modules
 *   array of module objects
 * @param bool $status
 *   boolean value for the module status.
 * @param array $parents
 *   array of additional parents to set to the form elements.
 * @return array
 *   form array with checkboxes
 */
function _master_modules_overview_modules_tableselect_form($modules, $status, $parents = array()) {

  // Provide the module list als tableselect.
  $form['#theme'] = 'master_tableselect';
  $form['#header'] = array('', t('Module'), t('Description'));
  $form['#columns'] = array('checkbox', 'name', 'description');
  $form['#empty'] = t('No modules.');

  foreach ($modules as $module) {
    // Provide checkboxes and description modules.
    if ($module->status == $status) {
      $form[$module->name]['checkbox'] = array(
        '#type' => 'checkbox',
        '#parents' => array_merge($parents , array($module->name)),
      );
      $form[$module->name]['name'] = array(
        '#value' => check_plain($module->info['name']),
      );
      $form[$module->name]['description'] = array(
        '#value' => filter_xss_admin($module->info['description']),
      );
    }
  }
  return $form;
}

/**
 * Provide a checkable tabelselect list of features modules.
 *
 * @param array $features
 *   array of feature/module objectes
 * @param array $parents
 *   array of additional parents to set to the form elements.
 * @return array
 *   form array with checkboxes
 */
function _master_modules_overview_features_tableselect_form($features, $parents = array()) {

  // Provide the module list als tableselect.
  $form['#theme'] = 'master_tableselect';
  $form['#header'] = array('', t('Feature'), t('Storage'), t('Revertable components'), t('All components'));
  $form['#columns'] = array('checkbox', 'name', 'storage', 'revertable_components', 'components');
  $form['#empty'] = t('There are no features in this master.');

  foreach ($features as $feature) {
    // Provide checkboxes and description for inactive modules.
    // Show a fieldset for Features (if module is a feature and dependency of the master).
    if ($feature->status) {

      // Build description as additional information, including the
      // storage status of the feature and a list of its components
      $storage = features_get_storage($feature->name);
      $path = "admin/build/features/{$feature->name}";
      $components = features_get_component_states(array($feature->name), FALSE);
      $revertable_components = array_keys(array_filter($components[$feature->name]));
      //$default_components = array_diff(array_keys($components[$module]), $revertable_components);
      $form[$feature->name]['checkbox'] = array(
        '#type' => 'checkbox',
        '#parents' => array_merge($parents , array($feature->name)),
      );
      $form[$feature->name]['name'] = array(
        '#value' => check_plain($feature->info['name']),
      );
      $form[$feature->name]['storage'] = array(
        '#value' => theme('features_storage_link', $storage, NULL, $path),
      );
      $form[$feature->name]['revertable_components'] = array(
        '#value' => theme('features_component_list', $revertable_components),
      );
      $form[$feature->name]['components'] = array(
        '#value' => theme('master_features_components', $feature),
      );
    }
  }
  return $form;
}


/**
 * Form submission for master module enabling.
 */
function master_modules_master_submit($form, &$form_state) {

  $to_enable = array_keys(array_filter($form_state['values']['dependent']['modules']));

  // @TODO: check dependencies.
  // @TODO: add more from system_modules_submit().

  // Enable modules.
  if (!empty($to_enable)) {
    $stringlist = implode(', ', $to_enable);
    module_enable($to_enable);
    drupal_set_message(t('Modules %list should now be enabled.', array('%list' => $stringlist)), 'notice');
  }
}

/**
 * Form submission for dependent module enabling.
 */
function master_modules_enable_submit($form, &$form_state) {

  $to_enable = array_keys(array_filter($form_state['values']['dependent']['modules']));

  // @TODO: check dependencies.
  // @TODO: add more from system_modules_submit().

  // Enable modules.
  if (!empty($to_enable)) {
    $stringlist = implode(', ', $to_enable);
    module_enable($to_enable);
    drupal_set_message(t('Modules %list should now be enabled.', array('%list' => $stringlist)), 'notice');
  }
}

/**
 * Form submission for master module enabling.
 */
function master_modules_disable_submit($form, &$form_state) {
  $to_disable = array_keys(array_filter($form_state['values']['independent']['modules']));

  // @TODO: check dependencies to other modules.
  // @TODO: add more from system_modules_submit().

  // disable modules.
  if (!empty($to_disable)) {
    $stringlist = implode(', ', $to_disable);
    module_disable($to_disable);
    drupal_flush_all_caches();
    drupal_set_message(t('Modules %list should now be disabled.', array('%list' => $stringlist)), 'notice');
    drupal_set_message(t('Caches cleared.'));
  }
  return $to_disable;
}

/**
 * Form submission to revert selected features.
 */
function master_modules_features_revert_submit($form, &$form_state) {

  $to_revert = array_keys(array_filter($form_state['values']['features']['modules']));

  $items = array();
  foreach ($to_revert as $module) {
    $states = features_get_component_states(array($module), FALSE);
    $components = array_keys(array_filter($states[$module]));
    drupal_set_message(t('Reverted components for <strong>@module</strong> of types: <strong>@components</strong> .', array('@components' => implode(', ', $components), '@module' => $module)));
    $items[$module] = $components;
  }

  features_revert($items);
}

/**
 * Helper function for array_filter, to retrieve active modules from an object
 * array.
 *
 * @param object $module
 *   a module object
 * @return bool
 *   TRUE if it is an enabled module, FALSE else
 */
function _master_array_filter_module_enabled($module) {
  return ($module->type == 'module' && $module->status);
}